package HoursToolWeb;
use Moose;
use namespace::autoclean;

use Catalyst::Runtime 5.80;

# Set flags and add plugins for the application.
#
# Note that ORDERING IS IMPORTANT here as plugins are initialized in order,
# therefore you almost certainly want to keep ConfigLoader at the head of the
# list if you're using it.
#
#         -Debug: activates the debug mode for very useful log messages
#   ConfigLoader: will load the configuration from a Config::General file in the
#                 application's home directory
# Static::Simple: will serve static files from the application's root
#                 directory

use Catalyst qw/
    -Debug
    ConfigLoader
    Static::Simple
    Session
    Session::Store::File
    Session::State::Cookie
    FormValidator
    Unicode::Encoding
    I18N
/;

extends 'Catalyst';

our $VERSION = '0.01';

# Configure the application.
#
# Note that settings in hourstoolweb.conf (or other external
# configuration file that you set up manually) take precedence
# over this when using ConfigLoader. Thus configuration
# details given here can function as a default configuration,
# with an external configuration file acting as an override for
# local deployment.

use HoursTool::Config;
my $main_config = HoursTool::Config->config( __PACKAGE__->path_to('../hourstool.conf') );

__PACKAGE__->config(
    name => 'HoursToolWeb',
    # Disable deprecated behavior needed by old applications
    disable_component_resolution_regex_fallback => 1,
    enable_catalyst_header => 1, # Send X-Catalyst header
);

__PACKAGE__->config( 'Model::HoursToolDB' => {
    connect_info => $main_config->{connect_info},
});



# Start the application
__PACKAGE__->setup();


sub uri_for_static {
    my $c = shift;
    return $c->uri_for( '/static', @_ );
}

sub redirect {
    my ( $c, $uri ) = @_;
    
    $c->res->body(undef);
    $c->res->redirect( $uri );
    $c->detach();
}


=head1 NAME

HoursToolWeb - Catalyst based application

=head1 SYNOPSIS

    script/hourstoolweb_server.pl

=head1 DESCRIPTION

[enter your description here]

=head1 SEE ALSO

L<HoursToolWeb::Controller::Root>, L<Catalyst>

=head1 AUTHOR

Todd Holbrook

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
