package HoursTool::Schema::Result::HoursForDay;

use base qw/DBIx::Class::Core/;

__PACKAGE__->table_class('DBIx::Class::ResultSource::View');
__PACKAGE__->table('faketable');
__PACKAGE__->mk_group_accessors('simple' => qw/days_cache/); # must use simple group

__PACKAGE__->add_columns(
    location_id => {
        data_type => 'integer',
    },
    location_hour_id => {
        data_type => 'integer',
    },
    range_id => {
        data_type => 'integer',
    },
    parent_id => {
        data_type => 'integer',
    },
    name => {
        data_type => 'text',
    },
    subname => {
        data_type => 'text',
    },
    description => {
        data_type => 'text',
    },
    priority => {
        data_type => integer,
    },
);
__PACKAGE__->set_primary_key('location_id');

__PACKAGE__->resultset_class('HoursTool::Schema::ResultSet::HoursForDay');

__PACKAGE__->result_source_instance->is_virtual(1);

__PACKAGE__->result_source_instance->view_definition(q[
    SELECT locations.name, locations.subname, locations.description, locations.parent_id, locations.id as location_id, lh.priority, lh.range_id AS range_id, lh.id AS location_hour_id FROM locations LEFT OUTER JOIN (
        SELECT DISTINCT ON (location_id) location_hours.id as id, location_hours.location_id, location_hours.priority, range_id FROM location_hours JOIN ranges ON ( ranges.id = location_hours.range_id ) WHERE start_date <= ? AND end_date >= ? AND ranges.active = 't' AND location_hours.active = 't' ORDER BY location_id, priority DESC
    ) lh ON ( locations.id = lh.location_id ) WHERE locations.active = 't'
]);

sub parent {
    my $self = shift;
    return $self->result_source->schema->resultset('Locations')->find({ id => $self->parent_id });
}


sub location {
    my $self = shift;
    return $self->result_source->schema->resultset('Locations')->find({ id => $self->location_id });
}

sub range {
    my $self = shift;
    return $self->result_source->schema->resultset('Ranges')->find({ id => $self->range_id });
}

sub day {
    my ( $self, $dow ) = @_;

    if ( !defined($self->days_cache) ) {
        my @days = $self->result_source->schema->resultset('Days')->search({ location_hour_id => $self->location_hour_id })->all;
        $self->days_cache( { map { $_->dow => $_ } @days } );
    }

    return $self->days_cache->{$dow};
}

sub to_json_hash {
    my $self = shift;

    # Build hashref to convert to JSON
    
    my %hash = $self->get_columns();

    return \%hash;
}


1;
