package HoursTool::Schema::Result::Locations;

use base 'HoursTool::Schema::SchemaBase';

__PACKAGE__->load_components(qw/  /);
__PACKAGE__->table('locations');

__PACKAGE__->add_columns(
    id => {
        data_type           => 'integer',
        is_nullable         => 0,
        is_auto_increment   => 1,
        default_value       => undef,
    },
    name => {
        data_type           => 'varchar',
        size                => 1024,
        is_nullable         => 0,
    },
    subname => {
        data_type           => 'varchar',
        size                => 1024,
        is_nullable         => 1,
    },
    parent_id => {
        data_type           => 'integer',
        is_nullable         => 1,
        default_value       => undef,
    },
    map_code => {
        data_type           => 'text',
    },
    description => {
        data_type           => 'text',
        is_nullable         => 1,
    },
    address => {
        data_type           => 'text',
        is_nullable         => 1,
    },
    phone => {
        data_type           => 'text',
        is_nullable         => 1,
    },
    rank => {
        data_type           => 'integer',
        is_nullable         => 0,
        default_value       => 0,
    },
    url => {
        data_type           => 'varchar',
        size                => 255,
        is_nullable         => 1,
    },
    note => {
        data_type           => 'text',
        is_nullable         => 1,
    },
    active => {
        data_type           => 'boolean',
        is_nullable         => 0,
        default_value       => 'true'
    },
);
__PACKAGE__->set_primary_key('id');

__PACKAGE__->has_many( 'location_hours' => 'HoursTool::Schema::Result::LocationHours', 'location_id' );
__PACKAGE__->belongs_to( 'parent' => 'HoursTool::Schema::Result::Locations', 'parent_id', { join_type => 'left' } );

sub to_json_hash {
    my $self = shift;

    # Build hashref to convert to JSON
    
    my %hash = $self->get_columns();

    return \%hash;
}

# Returns LocationHours with attached Ranges for the location which overlaps the 
# passed in start and end dates. The "_parent" version takes ranges from the parent
# and overlays the child ranges if they exist.

sub hours_in_range {
    my ( $self, $start, $end ) = @_;

    if ( $self->parent_id ) {
        return $self->hours_in_range_parent( $start, $end );
    }

    $start = $start->ymd if ( ref($start) && $start->can('ymd') );
    $end   = $end->ymd   if ( ref($end)   && $end->can('ymd') );

    my $search = { 
        'range.start_date' => { '<=' => $end },
        'range.end_date'   => { '>=' => $start },
        'range.active' => 't',
        'me.active' => 't',
    };

    my $hours_rs = $self->location_hours(
        $search,
        {
            join     => 'range',
            prefetch => 'range',
            order_by => 'priority, range.start_date',
        }
    );

    my @hours = $hours_rs->all;

    return \@hours;
}

sub hours_in_range_parent {
    my ( $self, $start, $end ) = @_;

    $start = $start->ymd if ( ref($start) && $start->can('ymd') );
    $end   = $end->ymd   if ( ref($end)   && $end->can('ymd') );

    my $parent = $self->parent;

    my $search = { 
        'range.start_date' => { '<=' => $end },
        'range.end_date'   => { '>=' => $start },
        'range.active' => 't',
        'me.active' => 't',
    };


    my $parent_rs = $parent->location_hours(
        $search,
        {
            join     => 'range',
            prefetch => 'range',
        }
    );

    my %hours = map { $_->range->id => $_ } $parent_rs->all;

    my $self_rs = $self->location_hours(
        $search,
        {
            join     => 'range',
            prefetch => 'range',
        }
    );

    foreach my $lh ( $self_rs->all ) {
        $hours{$lh->range->id} = $lh;
    }

    return [ sort { $a->priority <=> $b->priority or $a->range->start_date <=> $b->range->start_date } values %hours ];
}

1;
